Public Class EDS_HumidityTempProbeSHTP
	Public PrimaryDS2438 As DS2438

	Public Sub New(ByVal primaryDS2438 As DS2438)
		Me.PrimaryDS2438 = primaryDS2438
	End Sub

	Public Function getTemperature(ByVal lockid As String) As Single
		Dim localLock As Boolean
		Dim retVal As Single

		'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
		'neccessary to lock the bus to prevent interruption.
		If IsNothing(lockid) Then
			'The bus isn't already locked, so acquire a lock.
			lockid = Me.PrimaryDS2438.BusMaster.OW_GetLock().Data(0)
			localLock = True
		Else
			'The bus is already locked and being managed by the caller.
			localLock = False
		End If

		retVal = Me.PrimaryDS2438.GetTemperature(lockid)

		'Release the bus lock, if managed locally
		If localLock Then
			Me.PrimaryDS2438.BusMaster.OW_ReleaseLock()
		End If

		Return retVal
	End Function

	Public Function getRelativeHumidity(ByVal lockid As String) As Single
		Dim supplyVoltage As Single
		Dim humiditySensorVoltage As Single
		Dim temperature As Single
		Dim localLock As Boolean
		Dim sensorRH As Single
		Dim trueRH As Single

		'Since multiple commands may be issued to the 1-Wire bus to accomplish this function, it is
		'neccessary to lock the bus to prevent interruption.
		If IsNothing(lockid) Then
			'The bus isn't already locked, so acquire a lock.
			lockid = Me.PrimaryDS2438.BusMaster.OW_GetLock().Data(0)
			localLock = True
		Else
			'The bus is already locked and being managed by the caller.
			localLock = False
		End If

		temperature = Me.getTemperature(lockid)
		supplyVoltage = Me.PrimaryDS2438.GetVoltage(DS2438.AD_SOURCE_VDD, lockid)
		humiditySensorVoltage = Me.PrimaryDS2438.GetVoltage(DS2438.AD_SOURCE_VAD, lockid)

		'Release the bus lock, if managed locally
		If localLock Then
			Me.PrimaryDS2438.BusMaster.OW_ReleaseLock()
		End If

		sensorRH = (humiditySensorVoltage / supplyVoltage - 0.16) / 0.0062
		trueRH = sensorRH / (1.0546 - 0.00216 * temperature)

		Return trueRH

	End Function

End Class