' Imports OWInterface.OW_Response
' Imports SerialPort

	'Provides access to the HA5 RS232 / RS485 / TTL to 1-Wire Bus Master interface
	'manufactured by Embedded Data Systems, LLC.
	Public Class HA5
		Inherits EDS_OWInterface

		Public EnableCRC As Boolean		'Utilize CRC on serial communications
		Public HA5Address As String		'Single character a-z representing the HA5's address

		Const CMD_ADDRESS_DEVICE = "A"
		Const CMD_MATCH_DEVICE = "M"
		Const CMD_READ_FILE_RECORDS = "L"
		Const CMD_READ_PAGES = "G"
		Const CMD_RESET_BUS = "R"
		Const CMD_SEARCH = "S"
		Const CMD_WRITE_BIT = "B"
		Const CMD_WRITE_BLOCK = "W"
		Const CMD_WRITE_FILE_RECORD = "l"

		Private serialPort As EmbeddedDataSystems.SerialPort
		Private last1WireAddress As String

		Public Sub New(ByVal HA5Address As String, ByVal portName As String, ByVal baudRate As Long, ByVal enableCRC As Boolean)
			Me.HA5Address = HA5Address
			Me.EnableCRC = enableCRC
			serialPort = New EmbeddedDataSystems.SerialPort(portName, baudRate, serialPort.NOPARITY, 8, serialPort.ONESTOPBIT)
			serialPort.flushComIn()
		End Sub

		Public Overrides Function OW_AddressDevice(ByVal Address As String) As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList
			Dim ha5Response As String
			Dim ha5Request As String

			'Build the request
			ha5Request = HA5Address & CMD_ADDRESS_DEVICE & Address & vbCr

			'Get data from the HA5
			ha5Response = txHA5(ha5Request)
			Me.last1WireAddress = Address

			'Address data is returned for this request
			retData.Add(Left(ha5Response, 16))
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now
		End Function

		'Pretends to obtain an exclusive 1-Wire lock.
		Public Overrides Function OW_GetLock() As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList

			retData.Add("1234567890")
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now
		End Function

		Public Overrides Function OW_MatchROM() As OWInterface.OW_Response

			Dim retVal As New OW_Response
			Dim retData As New ArrayList
			Dim ha5Response As String
			Dim ha5Request As String

			'No required parameters

			'Build the request
			ha5Request = HA5Address & CMD_ADDRESS_DEVICE & last1WireAddress & vbCr

			'Get data from the HA5
			ha5Response = txHA5(ha5Request)

			'Address data is returned for this request
			retVal.Data = retData

			'Create stats / exception info
			retVal.ResponseTime = Now()

			'Return results
			Return retVal

		End Function

		Public Overrides Function OW_PowerDownBus() As OWInterface.OW_Response

		End Function

		Public Overrides Function OW_ReadBit() As OWInterface.OW_Response

		End Function

		Public Overrides Function OW_ReadFileRecords(ByVal StartRecord As Byte, Optional ByVal RecordsToRead As Byte = 1, Optional ByVal Address As String = "") As OWInterface.OW_Response

		End Function
		'Reads one or more memory pages from a 1-Wire device
		Public Overrides Function OW_ReadPages(ByVal StartPage As Byte, Optional ByVal PagesToRead As Byte = 1, Optional ByVal Address As String = "") As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList
			Dim ha5Response As String
			Dim ha5InitialRequest As String
			Dim ha5SubsequentRequest As String

			'Process the Optional parameters
			If Address.Length > 0 Then
				'Need to address the device first
				Me.OW_AddressDevice(Address)
			End If

			'Build the request, including any standard parameters
			ha5InitialRequest = HA5Address & CMD_READ_PAGES & "," & Hex2(1) & Hex2(StartPage) & vbCr
			ha5SubsequentRequest = HA5Address & CMD_READ_PAGES & vbCr

			'Get first page from the HA5
			ha5Response = txHA5(ha5InitialRequest)
			retData.Add(Left(ha5Response, Len(ha5Response) - 1))

			'Get subsequent pages from the HA5
			For t As Integer = 1 To PagesToRead - 1
				ha5Response = txHA5(ha5SubsequentRequest)
				retData.Add(Left(ha5Response, Len(ha5Response) - 1))
			Next
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now

			Return retVal
		End Function
		'Pretends to release an exclusive lock on the 1-Wire bus
		Public Overrides Function OW_ReleaseLock() As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList

			retData.Add("1234567890")
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now
		End Function

		'Resets the 1-Wire bus
		Public Overrides Function OW_ResetBus() As OWInterface.OW_Response
			txHA5(HA5Address & CMD_RESET_BUS & vbCr)
		End Function

		Public Overrides Function OW_SearchROM(Optional ByVal FamilyCode As Byte = 0, Optional ByVal Conditional As Boolean = False) As OWInterface.OW_Response
			Dim ha5Response As String
			Dim ha5Request As String
			Dim family As String
			Dim retData As New ArrayList
			Dim retVal As New OW_Response
			Dim tempString As String

			'Process the optional parameters
			If FamilyCode <> 0 Then
				family = Hex2(FamilyCode)
			End If

			'Perform the search
			If Not Conditional Then
				If FamilyCode = 0 Then
					'Performing a regular search
					ha5Request = HA5Address & CMD_SEARCH & ",01" & vbCr				'Performs initial search, returning only 1 address at a time
					Do
						ha5Response = txHA5(ha5Request)
						If ha5Response.Length() > 1 Then
							retData.Add(Left(ha5Response, 16))
						Else
							Exit Do
						End If
						ha5Request = HA5Address & CMD_SEARCH & vbCr					'Performs a subsequent search, returning only 1 address at a time
					Loop
				Else
					'Performing a regular family search
					ha5Request = HA5Address & "F" & family & vbCr				'Performs initial family search, returns 1 address
					Do
						ha5Response = txHA5(ha5Request)
						If ha5Response.Length() > 1 Then
							retData.Add(Left(ha5Response, 16))
						Else
							Exit Do
						End If
						ha5Request = HA5Address & "FM" & vbCr					'Performs a subsequent family search, returning 1 address at a time
					Loop
				End If
			End If
			If Conditional Then
				'Performing a conditional search
				ha5Request = HA5Address & "C" & ",01" & vbCr			 'Performs initial conditional search, returning only 1 address
				Do
					ha5Response = txHA5(ha5Request)
					If ha5Response.Length() > 1 Then
						If FamilyCode > 0 Then
							'User requested a Conditional Family Search, so now throw out the ROM-ID's that don't match the requested
							'family code
							If Right(ha5Response, 2) = family Then
								'Family code matches
								retData.Add(Left(ha5Response, 16))
							End If
						Else
							'No family code restriction requested
							retData.Add(Left(ha5Response, 16))
						End If
					Else
						Exit Do
					End If
					ha5Request = HA5Address & "C" & vbCr				'Performs a subsequent conditional search, returning 1 address at a time
				Loop
			End If

			retVal.Data = retData

			'Set stats / exception info
			retVal.ResponseTime = Now()

			Return retVal
		End Function

		Public Overrides Function OW_WriteBit(ByVal BitToWrite As Byte) As OWInterface.OW_Response

		End Function

		Public Overrides Function OW_WriteBlock(ByVal WhatToWrite As String, Optional ByVal Address As String = "") As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList

			Dim ha5Request As String
			Dim ha5Response As String

			'Process the Optional parameters
			If Address.Length > 0 Then
				'Need to address the device first
				Me.OW_AddressDevice(Address)
			End If

			'Build the request, including any standard parameters
			ha5Request = HA5Address & CMD_WRITE_BLOCK & Hex2(Len(WhatToWrite) / 2) & WhatToWrite & vbCr

			'Get data from the HA5
			ha5Response = txHA5(ha5Request)
			retData.Add(Left(ha5Response, Len(ha5Response) - 1))

			'Set stats / exception info
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now

			'Return results
			Return retVal
		End Function

		Public Overrides Function OW_WriteFileRecord(ByVal RecordNumber As Byte, ByVal WhatToWrite As String, Optional ByVal Address As String = "") As OWInterface.OW_Response

		End Function

		Public Overrides Function ResetInterface() As Boolean

		End Function
		'Transmits data to the HA5, and returns the response
		Private Function txHA5(ByVal whatToSend As String) As String
			Dim retVal As String
			Dim rx As String
			'TODO - Implement serial CRC functionality
			serialPort.tx(whatToSend)
			Do While InStr(retVal, Chr(13)) = 0
				rx = serialPort.rx()
				If rx.Length() = 0 Then
					'Timed out while reading from the port
					Exit Do
				Else
					retVal = retVal & rx
				End If
			Loop
			Return retVal
		End Function
	End Class