'Imports EmbeddedDataSystems.OWInterface.OW_Response
'Imports EmbeddedDataSystems.SerialPort
	Public Class HA7E
		Inherits EDS_OWInterface
		'Provides access to the HA7E RS232 to 1-Wire Bus Master interface
		'manufactured by Embedded Data Systems, LLC.

		Const CMD_ADDRESS_DEVICE = "A"
		Const CMD_MATCH_DEVICE = "M"
		Const CMD_READ_FILE_RECORDS = "L"
		Const CMD_READ_NEXT_FILE_RECORD = "l"
		Const CMD_READ_PAGES = "G"
		Const CMD_READ_NEXT_PAGE = "g"
		Const CMD_RESET_BUS = "R"
		Const CMD_SEARCH = "S"
		Const CMD_SEARCH_NEXT = "s"
		Const CMD_CONDITIONAL_SEARCH = "C"
		Const CMD_CONDITIONAL_SEARCH_NEXT = "c"
		Const CMD_FAMILY_SEARCH = "F"
		Const CMD_FAMILY_SEARCH_NEXT = "f"
		Const CMD_WRITE_BIT = "B"
		Const CMD_WRITE_BLOCK = "W"
		Const CMD_WRITE_FILE_RECORD = "U"

		Private serialPort As EmbeddedDataSystems.SerialPort
		Private last1WireAddress As String

		Public Sub New(ByVal portName As String)
			serialPort = New EmbeddedDataSystems.SerialPort(portName, 9600, serialPort.NOPARITY, 8, serialPort.ONESTOPBIT)
			serialPort.flushComIn()
		End Sub

		Public Overrides Function OW_AddressDevice(ByVal Address As String) As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList
			Dim ha7Response As String
			Dim ha7Request As String

			'Build the request
			ha7Request = CMD_ADDRESS_DEVICE & Address & vbCr

			'Get data from the HA7E
			Me.log("Addressing Device having ROMId " & Address & "." & vbCrLf)
			Me.incrementLogLevel()
			ha7Response = txHA7(ha7Request)
			Me.last1WireAddress = Address
			Me.decrementLogLevel()
			'Address data is returned for this request
			retData.Add(Left(ha7Response, 16))
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now
		End Function

		'Pretends to obtain an exclusive 1-Wire lock.
		Public Overrides Function OW_GetLock() As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList

			retData.Add("1234567890")
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now
			OW_GetLock = retVal
		End Function

		Public Overrides Function OW_MatchROM() As OWInterface.OW_Response

			Dim retVal As New OW_Response
			Dim retData As New ArrayList
			Dim ha7Response As String
			Dim ha7Request As String

			'No required parameters

			'Build the request
			ha7Request = CMD_ADDRESS_DEVICE & last1WireAddress & vbCr

			'Get data from the HA7
			Me.log("Matching Device having ROMId " & last1WireAddress & "." & vbCrLf)
			ha7Response = txHA7(ha7Request)

			'Address data is returned for this request
			retVal.Data = retData

			'Create stats / exception info
			retVal.ResponseTime = Now()

			'Return results
			Return retVal

		End Function

		Public Overrides Function OW_PowerDownBus() As OWInterface.OW_Response

		End Function

		Public Overrides Function OW_ReadBit() As OWInterface.OW_Response

		End Function

		Public Overrides Function OW_ReadFileRecords(ByVal StartRecord As Byte, Optional ByVal RecordsToRead As Byte = 1, Optional ByVal Address As String = "") As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList
			Dim ha7Response As String
			Dim ha7InitialRequest As String
			Dim ha7SubsequentRequest As String

			'Process the Optional parameters
			If Address.Length > 0 Then
				'Need to address the device first
				Me.OW_AddressDevice(Address)
			End If

			'Build the request, including any standard parameters
			ha7InitialRequest = CMD_READ_FILE_RECORDS & Hex2(StartRecord)
			ha7SubsequentRequest = CMD_READ_NEXT_FILE_RECORD

			'Get first record from the HA7
			Me.log("Reading " & CStr(RecordsToRead) & " records from Device having ROMId " & Address & ", starting at record " & CStr(StartRecord) & "." & vbCrLf)
			Me.log("Reading first record." & vbCrLf)
			ha7Response = txHA7(ha7InitialRequest)
			retData.Add(Left(ha7Response, Len(ha7Response) - 1))

			'Get subsequent records from the HA7
			For t As Integer = 1 To RecordsToRead - 1
				ha7Response = txHA7(ha7SubsequentRequest)
				Me.log("Reading subsequent record." & vbCrLf)
				retData.Add(Left(ha7Response, Len(ha7Response) - 1))
			Next
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now

			Return retVal

		End Function
		'Reads one or more memory pages from a 1-Wire device
		Public Overrides Function OW_ReadPages(ByVal StartPage As Byte, Optional ByVal PagesToRead As Byte = 1, Optional ByVal Address As String = "") As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList
			Dim ha7Response As String
			Dim ha7InitialRequest As String
			Dim ha7SubsequentRequest As String

			'Process the Optional parameters
			If Address.Length > 0 Then
				'Need to address the device first
				Me.OW_AddressDevice(Address)
			End If

			'Build the request, including any standard parameters
			ha7InitialRequest = CMD_READ_PAGES & Hex2(StartPage)
			ha7SubsequentRequest = CMD_READ_NEXT_PAGE

			'Get first page from the HA7
			Me.log("Reading " & CStr(PagesToRead) & " memory pages from Device having ROMId " & Address & ", starting at page " & CStr(StartPage) & "." & vbCrLf)
			Me.log("Reading first page." & vbCrLf)
			ha7Response = txHA7(ha7InitialRequest)
			retData.Add(Left(ha7Response, Len(ha7Response) - 1))

			'Get subsequent pages from the HA7
			For t As Integer = 1 To PagesToRead - 1
				ha7Response = txHA7(ha7SubsequentRequest)
				Me.log("Reading subsequent page." & vbCrLf)
				retData.Add(Left(ha7Response, Len(ha7Response) - 1))
			Next
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now

			Return retVal
		End Function
		'Pretends to release an exclusive lock on the 1-Wire bus
		Public Overrides Function OW_ReleaseLock() As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList

			retData.Add("1234567890")
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now
		End Function

		'Resets the 1-Wire bus
		Public Overrides Function OW_ResetBus() As OWInterface.OW_Response
			Me.log("Resetting 1-Wire bus." & vbCrLf)
			txHA7(CMD_RESET_BUS)
		End Function

		Public Overrides Function OW_SearchROM(Optional ByVal FamilyCode As Byte = 0, Optional ByVal Conditional As Boolean = False) As OWInterface.OW_Response
			Dim ha7Response As String
			Dim ha7Request As String
			Dim family As String
			Dim retData As New ArrayList
			Dim retVal As New OW_Response
			Dim tempString As String


			'Process the optional parameters
			If FamilyCode <> 0 Then
				family = Hex2(FamilyCode)
			End If

			'Perform the search
			If Not Conditional Then
				If FamilyCode = 0 Then
					'Performing a regular search
					Me.log("Performing standard device search." & vbCrLf)
					ha7Request = CMD_SEARCH				'Performs initial search, returning only 1 address at a time
					Do
						ha7Response = txHA7(ha7Request)
						If ha7Response.Length() > 1 Then
							retData.Add(Left(ha7Response, 16))
						Else
							Exit Do
						End If
						ha7Request = CMD_SEARCH_NEXT				 'Performs a subsequent search, returning only 1 address at a time
					Loop
				Else
					'Performing a regular family search
					Me.log("Performing family search." & vbCrLf)
					ha7Request = CMD_FAMILY_SEARCH & family				'Performs initial family search, returns 1 address
					Do
						ha7Response = txHA7(ha7Request)
						If ha7Response.Length() > 1 Then
							retData.Add(Left(ha7Response, 16))
						Else
							Exit Do
						End If
						ha7Request = CMD_FAMILY_SEARCH_NEXT				 'Performs a subsequent family search, returning 1 address at a time
					Loop
				End If
			End If
			If Conditional Then
				'Performing a conditional search
				Me.log("Performing conditional search." & vbCrLf)
				ha7Request = CMD_CONDITIONAL_SEARCH			'Performs initial conditional search, returning only 1 address
				Do
					ha7Response = txHA7(ha7Request)
					If ha7Response.Length() > 1 Then
						If FamilyCode > 0 Then
							'User requested a Conditional Family Search, so now throw out the ROM-ID's that don't match the requested
							'family code
							If Right(ha7Response, 2) = family Then
								'Family code matches
								retData.Add(Left(ha7Response, 16))
							End If
						Else
							'No family code restriction requested
							retData.Add(Left(ha7Response, 16))
						End If
					Else
						Exit Do
					End If
					ha7Request = CMD_CONDITIONAL_SEARCH_NEXT				'Performs a subsequent conditional search, returning 1 address at a time
				Loop
			End If

			retVal.Data = retData

			'Set stats / exception info
			retVal.ResponseTime = Now()

			Return retVal
		End Function

		Public Overrides Function OW_WriteBit(ByVal BitToWrite As Byte) As OWInterface.OW_Response

		End Function

		Public Overrides Function OW_WriteBlock(ByVal WhatToWrite As String, Optional ByVal Address As String = "") As OWInterface.OW_Response
			Dim retVal As New EmbeddedDataSystems.OWInterface.OW_Response
			Dim retData As New ArrayList

			Dim ha7Request As String
			Dim ha7Response As String

			'Process the Optional parameters
			If Address.Length > 0 Then
				'Need to address the device first
				Me.OW_AddressDevice(Address)
			End If

			'Build the request, including any standard parameters
			ha7Request = CMD_WRITE_BLOCK & Hex2(Len(WhatToWrite) / 2) & WhatToWrite & vbCr

			'Get data from the HA7
			Me.log("Writing block of data: " & WhatToWrite & "." & vbCrLf)
			Me.incrementLogLevel()
			ha7Response = txHA7(ha7Request)
			Me.decrementLogLevel()
			retData.Add(Left(ha7Response, Len(ha7Response) - 1))

			'Set stats / exception info
			retVal.Data = retData
			retVal.Exception_Code = "0"
			retVal.Exception_Description = "None"
			retVal.ResponseTime = Now

			'Return results
			Return retVal
		End Function

		Public Overrides Function OW_WriteFileRecord(ByVal RecordNumber As Byte, ByVal WhatToWrite As String, Optional ByVal Address As String = "") As OWInterface.OW_Response

		End Function

		Public Overrides Function ResetInterface() As Boolean

		End Function
		'Transmits data to the HA7, and returns the response
		Private Function txHA7(ByVal whatToSend As String) As String
			Dim retVal As String
			Dim rx As String
			'TODO - Implement serial CRC functionality
			MyBase.log("TX- " & whatToSend & vbCrLf)
			serialPort.tx(whatToSend)
			Do While InStr(retVal, Chr(13)) = 0
				rx = serialPort.rx()
				If rx.Length() = 0 Then
					'Timed out while reading from the port
					Exit Do
				Else
					retVal = retVal & rx
				End If
			Loop
			MyBase.log("RX- " & retVal & vbCrLf)
			Return retVal
		End Function
	End Class