Imports System.Net.GlobalProxySelection
Imports System.Text.RegularExpressions
'Imports EmbeddedDataSystems.OWInterface.OW_Response
'Provides access to the HA7Net Ethernet to 1-Wire Bus Master interface
'manufactured by Embedded Data Systems, LLC.
Public Class HA7Net
	Inherits EDS_OWInterface

	Const URL_ADDRESS_DEVICE = "/1Wire/AddressDevice.html"	 '/1Wire/AddressDevice.html
	Const URL_GET_1WIRE_LOCK = "/1Wire/GetLock.html"	 '/1Wire/GetLock.html
	Const URL_MATCH_DEVICE = "/1Wire/MatchDevice.html"	 '/1Wire/MatchDevice.html
	Const URL_POWER_DOWN_BUS = "/1Wire/PowerDownBus.html"	 '/1Wire/PowerDownBus.html
	Const URL_READ_BIT = "/1Wire/ReadBit.html"		'/1Wire/ReadBit.html
	Const URL_READ_FILE_RECORDS = "/1Wire/ReadFileRecords.html"	 '/1Wire/ReadFileRecords.html
	Const URL_READ_PAGES = "/1Wire/ReadPages.html"	 '/1Wire/ReadPages.html
	Const URL_RELEASE_1WIRE_LOCK = "/1Wire/ReleaseLock.html"	 '/1Wire/ReleaseLock.html
	Const URL_RESET_BUS = "/1Wire/Reset.html"		'/1Wire/Reset.html
	Const URL_SEARCH = "/1Wire/Search.html"		'/1Wire/Search.html
	Const URL_WRITE_BIT = "/1Wire/WriteBit.html"		'/1Wire/WriteBit.html
	Const URL_WRITE_BLOCK = "/1Wire/WriteBlock.html"	 '/1Wire/WriteBlock.html
	Const URL_WRITE_FILE_RECORD = "/1Wire/WriteFileRecord.html"	 '/1Wire/WriteFileRecord.html
	Const URL_READ_TEMPERATRE = "/1Wire/ReadTemperature.html"	'/1Wire/ReadTemperature.html
	Const URL_READ_DS18B20TEMPERATRE = "/1Wire/ReadDS18B20.html"	'/1Wire/ReadDS18B20Temperature.html

	Const TEST_URL_STYLESHEET = "/"	 'Static content for testing
	Const USER_NAME = "root"	 'For testing
	Const USER_PASSWORD = "Netsilicon"	 'For testing

	'Contains data that describes a single HA7Net found
	'by using the Multicast discovery mechanism.
	Public Class DiscoveryResponse
		Public Signature As String		 'HA for standard HA7Net models
		Public Command As Int32			'8001h for standard HA7Net models
		Public Port As Int32			'Port number the http server can be reached on
		Public SSLPort As Int32			'Port number the https server can be reached on
		Public SerialNumber As String		 'MAC address of the HA7Net
		Public DeviceName As String		 'User configurable device name returned by the HA7Net
		Public IPAddress As System.Net.IPAddress		 'IP Address of the HA7Net
	End Class
	Public HostName As String	 'Hostname of the HA7Net represented by this instance
	Public HTTPPort As Integer	 'Port number of the http server on the HA7Net represented by this instance
	Public HTTPSPort As Integer	 'Port number of the https server on the HA7Net represented by this instance
	Public UseSSL As Boolean		'Flag to indicate if all http communications with this HA7Net should be made via https

	'Parameters used to build the URL of requests made to the HA7Net
	Private Class HttpParameter
		Public Name As String
		Public Value As String
		'Default constructor for HttpParameter class
		Public Sub New(ByVal Name As String, ByVal Value As String)
			Me.Name = Name
			Me.Value = Value
		End Sub
	End Class
	Private LockID As String

	' Uses a multicast mechanism to discover any HA7Nets within
	' reach of a Multicast packet. This mechanism works by
	' transmitting a particularly formatted packet to the multicast
	' group 224.1.2.3 on port 4567. Any HA7Nets that hear the
	' packet will respond by transmitting a directed UDP packet
	' back to the client that sent the multicast packet. The packet
	' received from the HA7Nets contain enough information that the
	' client should be able to figure out how to communicate
	' directly to the HA7Net.
	' 
	' Returns
	' An ArrayList of HA7Net.DiscoveryResponse, which details the
	' information about each HA7Net that was discovered.           
	Shared Function DiscoverHA7Nets() As ArrayList
		Dim HA7Nets As New ArrayList
		Dim HA7NetResponse As DiscoveryResponse
		Dim multicastDestination As New System.Net.IPEndPoint(System.Net.IPAddress.Parse("224.1.2.3"), 4567)
		Dim receiveBytes(256) As Byte		 'Buffer to hold incoming network data
		Dim RemoteIpEndPoint As New System.Net.IPEndPoint(System.Net.IPAddress.Any, 0)
		Dim txPacket As Byte() = System.Text.ASCIIEncoding.ASCII.GetBytes("HA" & Chr(0) & Chr(1))
		Dim udpSocket As New System.Net.Sockets.Socket _
		 (Net.Sockets.AddressFamily.InterNetwork, Net.Sockets.SocketType.Dgram, Net.Sockets.ProtocolType.Udp)

		'Transmit the Multicast Packet
		udpSocket.SendTo(txPacket, multicastDestination)

		'Give the responses .750 seconds to come back
		System.Threading.Thread.Sleep(750)

		Do While udpSocket.Available()
			udpSocket.ReceiveFrom( _
			 receiveBytes, receiveBytes.Length, Net.Sockets.SocketFlags.None, RemoteIpEndPoint)
			HA7NetResponse = ParseMulticastResponse(receiveBytes)
			If Not (HA7NetResponse Is Nothing) Then
				'Found a valid response...
				'Store the IP address that this response came from
				HA7NetResponse.IPAddress = RemoteIpEndPoint.Address
				'Add this HA7Net to the list
				HA7Nets.Add(HA7NetResponse)
			End If
		Loop
		udpSocket.Close()
		Return HA7Nets
	End Function

	' Parses rawBytes looking for a valid response to a multicast discovery packet.
	' If a valid response is found, then returns a DiscoveryResponse object containing 
	' the data parsed from the response, which should contain information regarding how
	' to communicate directly with the HA7Net.
	' It rawBytes does not contain a valid response, then returns Nothing.
	Shared Function ParseMulticastResponse(ByVal rawBytes As Byte()) As DiscoveryResponse

		Dim deviceNameLength As Int16
		Dim retVal As DiscoveryResponse
		Dim serialNumberLength As Int16
		retVal = Nothing

		If (System.Text.Encoding.ASCII.GetString(rawBytes, 0, 2)) = "HA" Then
			If ((rawBytes(2) * 256) + (rawBytes(3))) = &H8001 Then
				'This seems like an HA7 response to the multicast discovery request
				retVal = New DiscoveryResponse
				retVal.Signature = (System.Text.Encoding.ASCII.GetString(rawBytes, 0, 2))
				retVal.Command = (rawBytes(2) * 256) + rawBytes(3)
				retVal.Port = (rawBytes(4) * 256) + rawBytes(5)
				retVal.SSLPort = (rawBytes(6) * 256) + rawBytes(7)

				retVal.SerialNumber = System.Text.Encoding.ASCII.GetString(rawBytes, 8, 12)
				'Check for null termination of serial number, and trim the return value to match
				serialNumberLength = InStr(retVal.SerialNumber, Chr(0)) - 1
				If (serialNumberLength > 0) Then
					retVal.SerialNumber = Left$(retVal.SerialNumber, serialNumberLength)
				End If

				retVal.DeviceName = System.Text.Encoding.ASCII.GetString(rawBytes, 20, 64)
				'Check for null termination of device name, and trim the return value to match
				deviceNameLength = InStr(retVal.DeviceName, Chr(0)) - 1
				If (deviceNameLength > 0) Then
					retVal.DeviceName = Left$(retVal.DeviceName, deviceNameLength)
				End If
			End If
		End If
		Return retVal
	End Function
	' Default HA7Net constructor
	' 
	' Parameters
	' HostName :  String representation of the hostname or IP address of
	'             the HA7Net represented by this instance.              
	Public Sub New(ByVal HostName As String, Optional ByVal httpPort As Integer = 80, Optional ByVal httpsPort As Integer = 243)
		Me.HostName = HostName
		Me.HTTPPort = httpPort
		Me.HTTPSPort = httpsPort
	End Sub

	' This command will reset the 1-Wire bus, and then select the
	' particular 1-Wire device on the 1-Wire bus that you want to
	' talk to. Generally, communications on the 1-Wire bus occur
	' between the bus master (HA7Net), and a single 1-Wire device.
	' Before you can have communications with that particular
	' device, you must select the device which accomplishes two
	' things:
	' 
	'       1. It tells the device you are addressing that you
	'          intend to communicate with it.
	'       2. All other devices drop off of the bus until the next
	'          bus reset.
	' 
	' Parameters
	' Address :  8 byte 1\-Wire ROM Id, represented as a 16 byte hex
	'            string. This address might have been previously discovered
	'            using the Search command, stored in a database from an
	'            initial system installation, etc.
	' 
	' Returns:
	' OW_Interface.OW_Response: Response data and meta-data.
	' Addresses Table that contains the single 8 byte 1-Wire ROM
	' address code. This value is stored in a text field named
	' Address_0, and is the same value that was passed in the
	' Address parameter.
	' 
	' Note
	' The HA7Net will reset the bus during the address, so an
	' explicit Reset is not neccessary prior to calling this.              
	Public Overrides Function OW_AddressDevice(ByVal Address As String) As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required address parameter
		parameterList.Add(New HttpParameter("Address", Address))

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_ADDRESS_DEVICE, parameterList)
		Me.log("Addressing device with ROMId " & Address & vbCrLf)
		Me.incrementLogLevel()
		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'Address data is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Address_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
			Me.log("RX: " & m.Groups("Value").ToString())
		Next
		Me.decrementLogLevel()
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal
	End Function
	' This command requests exclusive use of the 1-Wire bus from
	' the HA7Net.
	' 
	' 
	' 
	' Returns:
	' OW_Interface.OW_Response: Response data and meta-data.
	' LockIDs Table that contains the single 10 byte LockID.
	' This value is stored in a text field named LockID_0. Upon
	' successfull acquistion of the lock, this instance of the
	' HA7Net class will automatically reference the LockId when
	' making requests from the physical device. This behaviour will
	' continue until the lock is explicitly released by calling
	' OW_ReleaseLock().
	' 
	' Note
	' The physical layer of the 1-Wire MicroLan is designed such
	' that only one data request or operation can be made on the
	' 1-Wire bus at a time. It is also common for a single logical
	' transaction to span multiple 1-Wire requests. Given this
	' modal nature of the 1-Wire bus, and the multi-user nature of
	' the http interface, it is necessary for the HA7Net to provide
	' a means of concurrency and bus contention management. Concurrency
	' management is provided on the HA7Net via a traditional
	' locking mechanism wherein a client will request a lock on the
	' 1-Wire bus and no other clients will be allowed access to the
	' bus until the lock is either explicitly released or expires. The
	' default maximum idle lifetime of the locks can be configured
	' per HA7Net, and is discussed later in this document.
	' 
	' 
	' 
	' The general process for interacting with the HA7Net in a
	' multi-user environment looks like this:
	' 
	'  Client requests lock of the 1-Wire bus
	' 
	' \o http://HA7.Net/1-Wire/GetLock.html
	' 
	'  HA7Net responds with a new LockId when the bus is available
	' 
	'  Client performs transactions while referencing the LockID
	' 
	' \o http://HA7.Net/1-Wire/Temperature.html?LockID=xxxxxxx&Address=1234567812345678
	' 
	' \o http://HA7.Net/1-Wire/Search?LockID=xxxxxx&FamilyCode=10&Conditional=True
	' 
	'  Client releases previously acquired lock
	' 
	' \o http://HA7.Net/1-Wire/ReleaseLock.html?LockID=xxxxxxx
	' 
	' For the duration of the lock, no other clients will be
	' allowed to access the 1-Wire bus.
	' 
	' 
	' 
	' In summary, all 1-Wire activities are handled atomically
	' within the HA7Net so all requests made that do not reference
	' a LockID are guaranteed to have uninterrupted access to the
	' 1-Wire bus for the duration of that particular request. By
	' requesting and referencing an explicit lock, uninterrupted
	' access to the 1-Wire bus can be guaranteed across multiple
	' requests.
	'                                                                                  
	Public Overrides Function OW_GetLock() As OWInterface.OW_Response

		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'No required parameters

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_GET_1WIRE_LOCK, Nothing)

		Me.log("Requesting exclusive use of 1-Wire bus." & vbCrLf)
		Me.incrementLogLevel()
		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'LockID is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "LockID_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
			Me.LockID = m.Groups("Value").ToString()
			Me.log("RX: " & m.Groups("Value").ToString() & vbCrLf)
		Next
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)
		Me.decrementLogLevel()
		Me.log("Acquired Lock Id : " & Me.LockID & vbCrLf)
		'Return results
		Return retVal

	End Function
	' Releases a previously obtained lock on the 1-Wire bus from
	' the HA7Net
	' 
	' Returns
	' OW_Interface.OW_Response: Response data and meta-data.
	' 
	' See Also
	' OW_GetLock()                                              
	Public Overrides Function OW_ReleaseLock() As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim retVal As New OW_Response
		Dim retData As New ArrayList
		Dim parameterList As New ArrayList

		'Process Required parameters
		'The lock-id parameter will be automatically added by the buildHttpWebRequest function, below.

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_RELEASE_1WIRE_LOCK, parameterList)
		Me.log("Releasing exclusive bus lock id: " & Me.LockID & vbCrLf)
		Me.incrementLogLevel()
		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'Nothing is returned for this request
		Me.LockID = ""
		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)
		Me.decrementLogLevel()
		'Return results
		Return retVal
	End Function
	' This command is used to simultaneously reset the 1-Wire bus
	' and then reselect the 1-Wire device that was most recently
	' selected with the Address Device command.
	' 
	' Note
	' The previously selected address is maintained by the
	' hardware, and is global in context. This is likely to be safe
	' to call only inside the context of an explicit bus lock.
	' 
	' The HA7Net will reset the bus during the address, so an
	' explicit Reset is not neccessary prior to calling this.
	' 
	' See Also
	' OW_AddressDevice, OW_Search                                  
	Public Overrides Function OW_MatchROM() As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'No required parameters

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_MATCH_DEVICE, Nothing)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'Address data is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Address_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
		Next
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal
	End Function

	' This command is used to completely power down the 1-Wire bus.
	' During normal operation, the HA7Net holds the voltage level
	' \on the 1-Wire bus high. For power sensitive applications,
	' the Power Down Bus command can be called to reduce the 1-Wire
	' bus voltage to 0. Any 1-Wire activity from the HA7Net will
	' cancel the Power Down mode, although the suggested method for
	' coming out of power down is via the Reset command.           
	Public Overrides Function OW_PowerDownBus() As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'No required parameters

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_POWER_DOWN_BUS, Nothing)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'No data is returned for this request

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal
	End Function

	' This command is used to read a single bit from the 1-Wire bus
	' regardless of context.
	' 
	' Returns: OWInterface.OW_Response - Bits Table that
	' contains the bit read from the 1-Wire bus. This value is
	' stored in a text field named Bit_0, in the 'Data' field of
	' the OW_Response.                                             
	Public Overrides Function OW_ReadBit() As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'No required parameters

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_READ_BIT, Nothing)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'Bit data is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Bit_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
		Next
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal

	End Function

	' Used to read one or more consecutive TMEX formatted file
	' records from 1-Wire devices that have memory pages. The CRC16
	' is automatically checked on the records, and if any record is
	' not a valid Touch Memory File Record, an exception will be
	' returned and all good records up to the point of error will
	' be returned.
	' 
	' The byte count, continuation code, and CRC16 bytes of the
	' TMEX record are stripped from the file record before it is
	' returned. See Chapter 7 of the Book of DS19xx iButton
	' Standards from Dallas Semiconductor for a complete discussion
	' \of the Touch Memory File Structure.
	' 
	' If the Optional Address parameter is given, then the 1-Wire
	' bus will be reset and the device specified by Address will
	' be selected prior to reading the file records. If
	' RecordsToRead is not specified, then only one record will
	' be read. StartRecord must be specified.
	' 
	' Parameters
	' StartRecord :    Page number to reading the record from. Should be
	'                  an integral value between 0 and 255.
	' RecordsToRead :  Number of records to read. Defaults to 1 if not
	'                  specified
	' Address :        8 byte 1\-Wire ROM Id, represented as a 16 byte hex
	'                  string.                                            
	Public Overrides Function OW_ReadFileRecords(ByVal StartRecord As Byte, Optional ByVal RecordsToRead As Byte = 1, Optional ByVal Address As String = "") As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required parameters
		parameterList.Add(New HttpParameter("StartRecord", StartRecord))
		parameterList.Add(New HttpParameter("FileRecordsToRead", RecordsToRead))

		'Process the Optional parameters
		If Address <> "" Then
			parameterList.Add(New HttpParameter("Address", Address))
		End If

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_READ_FILE_RECORDS, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'Records data is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "FileRecord_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
		Next
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal
	End Function

	' This command is used to read one or more consequtive memory
	' pages from 1-Wire devices that have memory pages. If the
	' Optional Address parameter is given, then the 1-Wire bus
	' will be reset and the devices specified by Address will be
	' selected prior to reading the memory pages. If NumPages is
	' not specified, then only one page will be read. StartPage
	' must be specified.
	' 
	' Parameters
	' StartPage :    Page number to reading the from. Should be an
	'                integral value between 0 and 255.
	' PagesToRead :  Number of pages to read. Defaults to 1 if not
	'                specified
	' Address :      8 byte 1\-Wire ROM Id, represented as a 16 byte hex
	'                string.                                            
	Public Overrides Function OW_ReadPages(ByVal StartPage As Byte, Optional ByVal PagesToRead As Byte = 1, Optional ByVal Address As String = "") As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required parameters
		parameterList.Add(New HttpParameter("StartPage", StartPage))
		parameterList.Add(New HttpParameter("PagesToRead", PagesToRead))

		'Process the Optional parameters
		If Address <> "" Then
			parameterList.Add(New HttpParameter("Address", Address))
		End If

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_READ_PAGES, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'Page data is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Page_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
		Next
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal


	End Function

	' This command is used to reset the 1-Wire bus. Resetting the
	' bus returns all devices on the bus to the addressing mode,
	' where they wait for you to select the next device(s) that you
	' want to talk to.                                             
	Public Overrides Function OW_ResetBus() As OWInterface.OW_Response
		Dim retVal As New OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_RESET_BUS, Nothing)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'No real data returned for this request

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal

	End Function
	' Implements the 1-Wire search algorithm including each of the
	' regular search, family search, and conditional search
	' functionalities. This function is used to discover the 64-bit
	' ROM codes (addresses) of devices connected to the 1-Wire bus.
	' The search function can optionally restrict the returned list
	' \of addresses to include only those devices belonging to a
	' particular family, and/or those that are in a device defined
	' conditional state.
	' 
	' Parameters
	' FamilyCode :   One byte decimal number used to restrict the results
	'                to 1\-Wire devices belonging to the given family.
	' Conditional :  True if only devices in a conditional state are to be
	'                returned.
	' 
	' Returns
	' Addresses Table that contains a list of 8 byte 1-Wire ROM
	' address codes, with each ROM code residing a text field named
	' Address_x where x is a 0 based sequential integer. This is
	' stored as an ArrayList of Strings in the Data field of the
	' OWInterFace.OW_Response object.                                     
	Public Overrides Function OW_SearchROM(Optional ByVal FamilyCode As Byte = 0, Optional ByVal Conditional As Boolean = False) As OWInterface.OW_Response

		Dim htmlData As String
		Dim response As System.Net.HttpWebResponse
		Dim retData As New ArrayList
		Dim retVal As New OW_Response
		Dim parameterList As New ArrayList

		'Process the optional parameters
		If FamilyCode <> 0 Then
			parameterList.Add(New HttpParameter("FamilyCode", Hex2(FamilyCode)))
		End If
		If Conditional = True Then
			parameterList.Add(New HttpParameter("Conditional", "TRUE"))
		End If

		Dim request As System.Net.HttpWebRequest = buildHttpWebRequest(URL_SEARCH, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(request)

		'Copy the 1-Wire Addresses into a string array for returning to caller
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Address_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
		Next
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		Return retVal
	End Function

	' Used to write a single bit from the 1-Wire bus regardless of
	' context. Any response from a 1-Wire device on the bus is read
	' and returned.
	' 
	' Parameters
	' Bit :  0 or 1 bit to write to the 1\-Wire bus.
	' 
	' Returns
	' Bits Table that contains the bit read from the 1-Wire bus.
	' This value is stored in a text field named Bit_0. This is
	' stored as an ArrayList of Strings in the Data field of the
	' OWInterFace.OW_Response object.                              
	Public Overrides Function OW_WriteBit(ByVal BitToWrite As Byte) As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required parameters
		parameterList.Add(New HttpParameter("Bit", BitToWrite))

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_WRITE_BIT, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'Bit data is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Bit_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
		Next
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal



	End Function
	' Perhaps the most important HA7Net command, the Write Block
	' command allows you to write and read any raw data to the
	' 1-Wire bus under any context. This will allow you to
	' implement any device specific protocols.
	' 
	' If the optional Address parameter is specified, the HA7Net
	' will first reset the 1-Wire bus, then select the device
	' having that address prior to writing the block of data to the
	' bus. If the Address parameter is not present, then the
	' HA7Net will simply write the data and reads data back from
	' the bus without regard for the current state of the bus.
	' 
	' While a complete understanding of the 1-Wire electrical
	' interface is not necessary, there is an important concept for
	' you to understand in order to effectively use this command.
	' Every 1-Wire device can only talk back to the bus master
	' during a read cycle on the bus. The only time that read
	' cycles are created on the bus is immediately after each bit
	' that the host master writes to the bus. Therefore, if the
	' response you expect to read back from the 1-Wire bus is
	' longer than the data you intend to write to the bus, you must
	' pad the data you are writing with 1 bits in order to generate
	' the necessary time slots for the devices to write their
	' entire response back to you. In short, you can only read as
	' many bits from the 1-Wire bus as you write. For a complete
	' explanation of 1-Wire bus timing and communication, please
	' see chapter one of the Book of DS19xx iButton Standards from
	' Dallas Semiconductor.
	' 
	' 
	' 
	' Parameters
	' Address :      8 byte 1\-Wire ROM Id, represented as a 16 byte
	'                hex string.
	' WhatToWrite :  1\-28 bytes of data formatted as Hex.
	' 
	' Returns
	' ResultData Table that contains the data read back from the
	' 1-Wire bus. This value is stored in a text field named
	' ResultData_0. This is stored as an ArrayList of Strings in
	' the Data field of the OWInterFace.OW_Response object.         
	Public Overrides Function OW_WriteBlock(ByVal WhatToWrite As String, Optional ByVal Address As String = "") As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required parameters
		parameterList.Add(New HttpParameter("Data", WhatToWrite))

		'Process the Optional parameters
		If Address <> "" Then
			parameterList.Add(New HttpParameter("Address", Address))
		End If
		Me.log("Writing block of data: " & WhatToWrite & vbCrLf)
		Me.incrementLogLevel()
		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_WRITE_BLOCK, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'ResultData is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "ResultData_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34))
		Dim m As Match
		For Each m In re.Matches(htmlData)
			retData.Add(m.Groups("Value").ToString())
			Me.log("RX: " & (m.Groups("Value").ToString()) & vbCrLf)
		Next
		Me.decrementLogLevel()
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal

	End Function

	' Used to write a TMEX formatted file record to the memory page
	' specified by RecordNumber. During the write process, the
	' HA7Net will automatically set the continuation pointer in the
	' \file record to the next memory page, which limits its use to
	' files having contiguous records. The HA7Net will always write
	' 28 bytes of data to the file record, even if less than 28
	' bytes are supplied. The record is validated as it is written
	' to the device, and an exception will be returned if the
	' record fails to write correctly.
	' 
	' Please see Chapter 7 of the Book of DS19xx iButton Standards
	' from Dallas Semiconductor for a complete discussion of the
	' Touch Memory File Structure.
	' 
	' If the Optional Address parameter is given, then the 1-Wire
	' bus will be reset and the device specified by Address will
	' be selected prior to reading the file records. StartRecord
	' must be specified.
	' 
	' Parameters
	' Address :       8 byte 1\-Wire ROM Id, represented as a 16 byte hex
	'                 string.
	' RecordNumber :  Memory page number to write the record to. Should
	'                 be a byte value from 0 to 255.
	' WhatToWrite :   1\-28 bytes of data formatted as Hex (56 hex
	'                 characters).                                       
	Public Overrides Function OW_WriteFileRecord(ByVal RecordNumber As Byte, ByVal WhatToWrite As String, Optional ByVal Address As String = "") As OWInterface.OW_Response
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required parameters
		parameterList.Add(New HttpParameter("RecordNumber", RecordNumber))
		parameterList.Add(New HttpParameter("Data", WhatToWrite))

		'Process the Optional parameters
		If Address <> "" Then
			parameterList.Add(New HttpParameter("Address", Address))
		End If

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_WRITE_FILE_RECORD, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'No data is returned for this request

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal

	End Function
	Public Overrides Function HL_ReadTemperature(ByVal address As String, ByVal llCallback As EmbeddedDataSystems.Dallas1WireDevice.LowLevelTemperatureCallBack) As Decimal
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required parameters
		parameterList.Add(New HttpParameter("Address_Array", address))

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_READ_TEMPERATRE, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'ResultData is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Temperature_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34) & "></td>.*Resolution")
		Dim m As Match
		For Each m In re.Matches(htmlData)
			Debug.WriteLine(m.Groups("Value").ToString())
			retData.Add(m.Groups("Value").ToString())
			Me.log("RX: " & (m.Groups("Value").ToString()) & vbCrLf)
		Next
		Me.decrementLogLevel()
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal.Data(0)

	End Function
	Public Overrides Function HL_ReadDS18B20Temperature(ByVal address As String, ByVal resolution As Integer, ByVal llCallback As EmbeddedDataSystems.Dallas1WireDevice.LowLevelDS18B20TemperatureCallBack) As Decimal
		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse
		Dim htmlData As String
		Dim parameterList As New ArrayList
		Dim retVal As New OW_Response
		Dim retData As New ArrayList

		'Process the required parameters
		parameterList.Add(New HttpParameter("DS18B20Request", "{" & address & "," & resolution.ToString() & "}"))

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(URL_READ_DS18B20TEMPERATRE, parameterList)

		'Get data from the HA7NET
		htmlData = txHA7(ha7NetRequest)

		'ResultData is returned for this request
		Dim re As New Regex("INPUT.*NAME=" & Chr(34) & "Temperature_(?<RecordNumber>(\d*))" & Chr(34) & "\B.*VALUE=" & Chr(34) & "(?<Value>(.*))" & Chr(34) & "></td>.*Resolution")
		Dim m As Match
		For Each m In re.Matches(htmlData)
			Debug.WriteLine(m.Groups("Value").ToString())
			retData.Add(m.Groups("Value").ToString())
			Me.log("RX: " & (m.Groups("Value").ToString()) & vbCrLf)
		Next
		Me.decrementLogLevel()
		retVal.Data = retData

		'Get stats / exception info
		ParseResultMetaData(retVal, htmlData)

		'Return results
		Return retVal.Data(0)

	End Function

	Public Overrides Function ResetInterface() As Boolean

	End Function
	Private Function buildHttpWebRequest(ByVal url As String, ByVal parameters As ArrayList) As System.Net.HttpWebRequest
		Dim uri As String
		Dim port As Integer
		Dim httpWebReqeust As System.Net.HttpWebRequest

		If UseSSL Then
			uri = "https://"
			port = HTTPSPort
		Else
			uri = "http://"
			port = HTTPPort
		End If
		uri = uri & HostName & ":" & CStr(port)

		If parameters Is Nothing Then
			parameters = New ArrayList
		End If

		'Append the lockid to the parameter list, if one exists
		If Me.LockID <> "" Then
			Dim lockParameter As New HttpParameter("LockID", Me.LockID)
			parameters.Add(lockParameter)
		End If

		'Append any optional parameters to the URL
		Dim firstParameter As Boolean = True
		For Each parameter As HttpParameter In parameters
			If firstParameter Then
				url = url & "?"
				firstParameter = False
			Else
				url = url & "&"
			End If
			url = url & parameter.Name & "=" & parameter.Value
		Next

		'Create the HttpWebRequest
		httpWebReqeust = System.Net.WebRequest.Create(uri & url)
		'httpWebReqeust.Credentials = New System.Net.NetworkCredential(USER_NAME, USER_PASSWORD)
		'httpWebReqeust.KeepAlive = False
		'httpWebReqeust.ProtocolVersion = System.Net.HttpVersion.Version10
		'httpWebReqeust.Proxy = System.Net.GlobalProxySelection.GetEmptyWebProxy()

		httpWebReqeust.Timeout = 60000
		Return httpWebReqeust
	End Function
	Private Function getHTML(ByVal Response As System.Net.HttpWebResponse) As String
		Dim html As String
		Dim chars As Char()
		'Extracts the HTML from the httpWebResponse object
		Dim responseStream As System.IO.Stream = Response.GetResponseStream()
		Dim responseStreamReader As New System.IO.StreamReader(responseStream)

		html = responseStreamReader.ReadToEnd()

		responseStreamReader.Close()

		Return html
	End Function

	Private Function ParseResultMetaData(ByRef retVal As OW_Response, ByRef htmlData As String)
		'Parses the result time, and any exceptions from the response, and places these
		'values into the OW_Response structure.
		Dim re As Regex
		Dim m As Match
		Dim epochTime As New System.DateTime(1970, 1, 1)

		'Calculate response time, and store into the OW_Response structure
		re = New Regex("INPUT.*NAME=" & Chr(34) & "Completed_(?<RecordNumber>(\d*))\x22\B.*VALUE=\x22(?<Value>(\d*))\x22")
		m = re.Match(htmlData)
		If Not m Is Nothing Then
			retVal.ResponseTime = epochTime.Add(New TimeSpan(0, 0, CLng(m.Groups("Value").ToString())))
		End If

		'Parse and store Exception Data
		'Code
		re = New Regex("INPUT.*NAME=\x22Exception_Code_(?<RecordNumber>(\d*))\x22\B.*VALUE=\x22(?<Value>(\d*))\x22\sSize")
		m = re.Match(htmlData)
		If Not m Is Nothing Then
			retVal.Exception_Code = m.Groups("Value").ToString()
		End If
		'Description
		re = New Regex("INPUT.*NAME=\x22Exception_String_(?<RecordNumber>(\d*))\x22\B.*VALUE=\x22(?<Value>(.*))\x22\sSize")
		m = re.Match(htmlData)
		If Not m Is Nothing Then
			retVal.Exception_Description = m.Groups("Value").ToString()
		End If

	End Function
	'Makes the request to the HA7Net and returns the HTML response.
	Private Function txHA7(ByVal whatToSend As System.Net.HttpWebRequest) As String
		Dim response As System.Net.HttpWebResponse
		Dim html As String
		Me.log("Getting URL: " & whatToSend.Address.PathAndQuery & vbCrLf)

		response = CType(whatToSend.GetResponse(), System.Net.HttpWebResponse)
		html = getHTML(response)
		'response.Close()

		Return html

	End Function

	Public Sub GetStyleSheet()

		Dim ha7NetRequest As System.Net.HttpWebRequest
		Dim ha7NetResponse As System.Net.HttpWebResponse

		'No required parameters

		'Build the http request, including any standard parameters
		ha7NetRequest = buildHttpWebRequest(TEST_URL_STYLESHEET, Nothing)
		txHA7(ha7NetRequest)
	End Sub
End Class